import os
import _winreg as winreg
import itertools
import addonHandler

# splitext is used to strip the .pendingInstall extension.
ADDON_DIR = os.path.abspath(os.path.splitext(os.path.dirname(__file__))[0])
SURFACE_DLL_DIR = os.path.join(ADDON_DIR, "surface")

SURFACE_ARCHIND_REG_KEYS = {
	r"SOFTWARE\Classes\TypeLib\{FAA7065B-1F88-4C93-8DBD-DDB7930D54F0}": {
		"1.0": {
			"": "JSonar 1.0 Type Library",
			"0": {
				"win32": {
					"": "{dll}",
				},
			},
			"FLAGS": {
				"": "0",
			},
			"HELPDIR": {
				"": "{dllDir}",
			},
		},
	},
	r"SOFTWARE\Classes\CakewalkControlSurfaces\{C92C1EE1-2A2C-4E05-9380-7FC83EE2602F}": {
		"Description": "JSonar Control Surface",
		"HelpFilePath": "",
	},
	r"SOFTWARE\Classes\JSonar.ControlSurface": {
		"": "JSonar Control Surface",
		"CLSID": {
			"": "{{C92C1EE1-2A2C-4E05-9380-7FC83EE2602F}}",
		},
		"CurVer": {
			"": "JSonar.ControlSurface.1",
		},
	},
	r"SOFTWARE\Classes\JSonar.ControlSurface.1": {
		"": "JSonar Control Surface",
		"CLSID": {
			"": "{{C92C1EE1-2A2C-4E05-9380-7FC83EE2602F}}",
		},
	},
}
SURFACE_ARCHDEP_REG_KEYS = {
	r"SOFTWARE\Classes\CLSID\{C92C1EE1-2A2C-4E05-9380-7FC83EE2602F}": {
		"": "JSonar Control Surface",
		"Control": {},
		"InprocServer32": {
			"": "{dll}",
			"ThreadingModel": "Both",
		},
		"Insertable": {},
		"MiscStatus": {
			"": "0",
			"1": {
				"": "132497",
			},
		},
		"ProgID": {
			"": "JSonar.ControlSurface.1",
		},
		"Programmable": {},
		"ToolboxBitmap32": {
			"": "{dll}, 102",
		},
		"TypeLib": {
			"": "{{FAA7065B-1F88-4C93-8DBD-DDB7930D54F0}}",
		},
		"Version": {
			"": "1.0",
		},
		"VersionIndependentProgID": {
			"": "JSonar.ControlSurface",
		},
	},
}
SONAR_REG_KEYS = (
	r"Software\Cakewalk Music Software\Sonar Producer",
)

def installRegKeys(key, values, substitutions={}, extraSam=0):
	for valName, val in values.iteritems():
		if isinstance(val, dict):
			# This is a sub-key.
			with winreg.CreateKeyEx(key, valName, 0, winreg.KEY_ALL_ACCESS | extraSam) as subKey:
				installRegKeys(subKey, val, substitutions)
			continue
		winreg.SetValueEx(key, valName, None,
			winreg.REG_SZ,
			val.format(**substitutions))

def removeRegKeys(key, values, extraSam=0):
	for valName, val in values.iteritems():
		if not isinstance(val, dict):
			# We don't need to remove values,
			# as they'll be deleted along with their containing key.
			# (There shouldn't ever be values at the top level.)
			continue
		try:
			subKey = winreg.OpenKey(key, valName, 0, winreg.KEY_ALL_ACCESS | extraSam)
		except WindowsError:
			pass
		else:
			with subKey:
				removeRegKeys(subKey, val)
		try:
			if extraSam:
				winreg.DeleteKeyEx(key, valName, winreg.KEY_ALL_ACCESS | extraSam)
			else:
				winreg.DeleteKey(key, valName)
		except WindowsError:
			pass

def checkHasX64():
	return "PROCESSOR_ARCHITEW6432" in os.environ

def onInstall():
	# Install the surface.
	hasX64 = checkHasX64()
	substitutions = {"dll": os.path.join(SURFACE_DLL_DIR, "jsonar.dll"),
		"dllDir": SURFACE_DLL_DIR}
	installRegKeys(winreg.HKEY_CURRENT_USER, SURFACE_ARCHIND_REG_KEYS, substitutions)
	installRegKeys(winreg.HKEY_CURRENT_USER, SURFACE_ARCHDEP_REG_KEYS, substitutions,
		extraSam=winreg.KEY_WOW64_32KEY if hasX64 else 0)
	if hasX64:
		substitutions["dll"] = os.path.join(SURFACE_DLL_DIR, "jsonar_x64.dll")
		installRegKeys(winreg.HKEY_CURRENT_USER, SURFACE_ARCHDEP_REG_KEYS, substitutions,
			extraSam=winreg.KEY_WOW64_64KEY)

	# Enable Sonar accessibility mode for all installed versions of Sonar.
	for keyName in SONAR_REG_KEYS:
		with winreg.OpenKey(winreg.HKEY_CURRENT_USER, keyName) as key:
			for index in itertools.count():
				try:
					keyName = winreg.EnumKey(key, index)
				except WindowsError:
					break
				if not keyName[0].isdigit():
					# Not for a Sonar version.
					continue
				with winreg.OpenKey(key, keyName, 0, winreg.KEY_ALL_ACCESS) as versionKey:
					winreg.SetValueEx(versionKey, "AccessibilityMode", None, winreg.REG_DWORD, 2)

def onUninstall():
	if os.path.isdir(ADDON_DIR + addonHandler.ADDON_PENDINGINSTALL_SUFFIX):
		# This is an update, so don't do anything here.
		return
	# Remove the surface.
	removeRegKeys(winreg.HKEY_CURRENT_USER, SURFACE_ARCHIND_REG_KEYS)
	if checkHasX64():
		for extraSam in (winreg.KEY_WOW64_32KEY, winreg.KEY_WOW64_64KEY):
			removeRegKeys(winreg.HKEY_CURRENT_USER, SURFACE_ARCHDEP_REG_KEYS, extraSam=extraSam)
	else:
		removeRegKeys(winreg.HKEY_CURRENT_USER, SURFACE_ARCHDEP_REG_KEYS)
